<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use \Zeedhi\Framework\Util\Functions;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Request;
use Teknisa\MAN\Services\RNFService;
use Teknisa\MAN\Utilities;
use \Exception as Exception;

class RNFController {
    protected $rnfService;
    protected $utilService;

    const CUPOM_PROCESSADO = "Cupons processados com sucesso para geração de RNF!";
    const SEM_DADOS = "Sem dados para processamento!";

    public function __construct(RNFService $rnfService, Utilities\Util $utilService) {
        $this->rnfService = $rnfService;
        $this->utilService = $utilService;
    }

    private function buildParams($conditions) {
        $result = [];

        if(!Functions::arrayKeyExists('CDFILIAL', $conditions)) {
            $result['TCDFILIAL'] = 'T';
            $result['CDFILIAL'] = '';
        } else {
            $result['TCDFILIAL'] = '';
            
            if(is_array($conditions['CDFILIAL'])) {
                $result['CDFILIAL'] = "'" . implode("', '", $conditions['CDFILIAL']) . "'";
            } else {
                $result['CDFILIAL'] = "'" . $conditions['CDFILIAL'] . "'";
            }
        }

        if(Functions::arrayKeyExists('DATARNF', $conditions)) {
            $result['P_DTINI'] = $conditions['DATARNF'][0];
            $result['P_DTFIN'] = $conditions['DATARNF'][1];
        }

        if((Functions::arrayKeyExists('RNFINI', $conditions)) && (Functions::arrayKeyExists('RNFFIN', $conditions))){
            $result['P_NUMINI'] = $conditions['RNFINI'];
            $result['P_NUMFIN'] = $conditions['RNFFIN'];
        } else {
            $result['P_NUMINI'] = NULL;
            $result['P_NUMFIN'] = NULL;
        }

        if(Functions::arrayKeyExists('CDOPERADOR', $conditions)) {
            $result['P_CDOPERADOR'] = $conditions['CDOPERADOR'];
        }

        return $result;
    }

    private function validaConditions($conditions) {
        if((isset($conditions['CDFILIAL'])) && (!empty($conditions['CDFILIAL'])) &&
           (isset($conditions['P_DTINI']))  && (!empty($conditions['P_DTINI']))  &&
           (isset($conditions['P_DTFIN']))  && (!empty($conditions['P_DTFIN']))) {
            return true;
        } else {
            return false;
        }
    }

    public function findFiliais(DTO\Request\Filter $request, DTO\Response $response) {
        $result = [];
        
        $conditions = $this->utilService->getParamsAsArray($request);
        $conditions = $this->buildParams($conditions);
        
        if($this->validaConditions($conditions)) {
            $result = $this->rnfService->findFiliais($conditions);
            
            $response->addDataSet(new DataSet("sdx35000_geracaoRNF"), $result);
        }

        return $result;
    }

    public function findCabecalho(DTO\Request\Filter $request, DTO\Response $response) {
        $result = [];
        
        $conditions = $this->utilService->getParamsAsArray($request);
        $conditions = $this->buildParams($conditions);
        
        if($this->validaConditions($conditions)) {
            $result = $this->rnfService->findCabecalho($conditions);
            
            $response->addDataSet(new DataSet("rnf_cabecalhos"), $result);
        }

        return $result;
    }

    public function findNotas(DTO\Request\Filter $request, DTO\Response $response) {
        $result = [];
        
        $conditions = $this->utilService->getParamsAsArray($request);
        $conditions = $this->buildParams($conditions);
        
        if($this->validaConditions($conditions)) {
            $result = $this->rnfService->findNotas($conditions);
            
            $response->addDataSet(new DataSet("rnf_notas"), $result);
        }

        return $result;
    }

    public function findItens(DTO\Request\Filter $request, DTO\Response $response) {
        $result = [];
        
        $conditions = $this->utilService->getParamsAsArray($request);
        $conditions = $this->buildParams($conditions);
        
        if($this->validaConditions($conditions)) {
            $result = $this->rnfService->findItens($conditions);
            
            $response->addDataSet(new DataSet("rnf_itens"), $result);
        }

        return $result;
    }

    public function findImpostos(DTO\Request\Filter $request, DTO\Response $response) {
        $result = [];
        
        $conditions = $this->utilService->getParamsAsArray($request);
        $conditions = $this->buildParams($conditions);
        
        if($this->validaConditions($conditions)) {
            $result = $this->rnfService->findImpostos($conditions);
            
            $response->addDataSet(new DataSet("rnf_impostos"), $result);
        }

        return $result;
    }

    public function gerarRNF(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();

            if(!Functions::arrayKeyExists('CDFILIAL', $row)) {
                $row['TCDFILIAL'] = 'T';
                $row['P_CDFILIAL'] = '';
            } else {
                $row['TCDFILIAL'] = '';
                
                if(is_array($row['CDFILIAL'])) {
                    $row['P_CDFILIAL'] = "'" . implode("', '", $row['CDFILIAL']) . "'";
                } else {
                    $row['P_CDFILIAL'] = "'" . $row['CDFILIAL'] . "'";
                }
            }

            $result = $this->rnfService->gerarRNF($row);

            if(!empty($result)) {
                $response->addMessage(new Message(self::CUPOM_PROCESSADO, Message::TYPE_MESSAGE));
            } else {
                $response->addMessage(new Message(self::SEM_DADOS, Message::TYPE_MESSAGE));
            }

            $response->addDataSet(new DataSet("gerarRNF", $result));
        } catch(Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function verificaIntegracao(DTO\Request $request, DTO\Response $response) {
        $result = [];

        $result = $this->rnfService->verificaIntegracao();

        $response->addMessage(new Message($result, Message::TYPE_MESSAGE));

        $response->addDataSet(new DataSet("verificaIntegracao"), $result);
    }
}